'''
Based on https://www.tensorflow.org/beta/guide/keras/saving_and_serializing

    Training a custom model created using tensorflow 2.0.
    This example is using the MNIST database of handwritten digits
    (http://yann.lecun.com/exdb/mnist/)
    Author: tensorflow
    Project: https://github.com/tensorflow/docs
    Modifications:

*****************************************************************
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2019. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*****************************************************************
'''

import mnist_custom_model
import argparse
import os
import ddl
import tensorflow as tf
import numpy as np

def main():
    model = mnist_custom_model.get_model()

    (x_train, y_train), (x_test, y_test) = tf.keras.datasets.mnist.load_data()
    x_train = x_train.reshape(60000, 784).astype('float32') / 255
    x_test  = x_test.reshape(10000, 784).astype('float32') / 255

    #Split dataset for DDL purposes
    x_train = np.array_split(x_train, ddl.size())[ddl.rank()]
    y_train = np.array_split(y_train, ddl.size())[ddl.rank()]


    model.compile(loss='sparse_categorical_crossentropy',
                  optimizer=tf.keras.optimizers.RMSprop(),
                  metrics=['accuracy'])

    history = model.fit(x_train, y_train,
                        batch_size=FLAGS.batch_size,
                        epochs=FLAGS.epochs)

    #Save the trained model
    if ddl.rank() == 0:
        print ("Testing Accuracy:", model.evaluate(x_test, y_test)[-1])
        model_dir = FLAGS.saved_model_path + "/mnist-custom-training-weights"
        os.makedirs(model_dir, exist_ok=True)
        print ("Saving trained model to ", model_dir)
        model.save_weights(model_dir, save_format='tf')

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--epochs', type=int, default=2)
    parser.add_argument('--batch_size', type=int, default=64)
    parser.add_argument('--saved_model_path', type=str, default="/tmp/mnist-model")

    FLAGS, unparsed = parser.parse_known_args()

    main()
